pub struct SignalModel {
pub current_value: u64,
pub change_observed: bool,
pub pending: bool,
pub notified: bool,
}Expand description
Erased one-listener logical view used by fused Signal realizations.
Fields
current_value: u64Current signal value.
change_observed: boolWhether any actual change has been retained.
pending: boolWhether the selected listener trails the current change head.
notified: boolWhether the selected listener has observed the current change head.