#[non_exhaustive]pub enum ActuationError {
SeatOutOfRange,
PassComplete,
SeatAlreadyAllocated,
SeatUnallocated,
SeatAlreadyActuated,
PassIncomplete,
}Expand description
A disabled actuation transition.
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.
SeatOutOfRange
A seat index is outside the admitted seat universe.
PassComplete
The pass has already committed closure.
SeatAlreadyAllocated
The seat already holds a resource.
SeatUnallocated
The seat holds no resource.
SeatAlreadyActuated
The seat has already committed its effect.
PassIncomplete
At least one allocated seat has not committed its effect.