#[non_exhaustive]pub enum PropagationError {
NodeOutOfRange,
RoundAlreadyRunning,
RoundNotRunning,
NodeAlreadyUpdated,
RoundIncomplete,
PassTerminated,
PassStillRunning,
}Expand description
A disabled propagation transition.
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.
NodeOutOfRange
A node index is outside the admitted graph.
RoundAlreadyRunning
A round is already running.
RoundNotRunning
The operation requires a running round.
NodeAlreadyUpdated
The node has already committed its update in this round.
RoundIncomplete
Not every node has committed an update.
PassTerminated
The pass is settled or has exhausted its iteration ceiling.
PassStillRunning
The pass is not yet settled and has not reached its ceiling.