#[non_exhaustive]pub enum QualityHierarchyError {
NodeOutOfRange,
ParentOutOfRange,
ChildOutOfRange,
LevelOutOfRange,
CostOutOfRange,
NodeNotIsolated,
SelfEdge,
EdgeAlreadyExists,
ChildAlreadyParented,
LevelOrderViolation,
CostOrderViolation,
}Expand description
A disabled quality-hierarchy transition.
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.
NodeOutOfRange
A node index is outside the admitted node set.
ParentOutOfRange
A proposed parent index is outside the admitted node set.
ChildOutOfRange
A proposed child index is outside the admitted node set.
LevelOutOfRange
A proposed level exceeds the hierarchy ceiling.
CostOutOfRange
A proposed cost exceeds the hierarchy ceiling.
NodeNotIsolated
Node-property updates require an isolated node.
SelfEdge
A node cannot be its own child.
EdgeAlreadyExists
The exact parent-child edge already exists.
ChildAlreadyParented
The proposed child already has a parent.
LevelOrderViolation
Parent level must strictly exceed child level.
CostOrderViolation
Parent cost must not exceed child cost.