#[non_exhaustive]pub enum SelectThenActuateError {
SeatOutOfRange,
CandidateOutOfRange,
PassComplete,
SeatAlreadyAllocated,
EffectAlreadyApplied,
SeatNotAllocated,
SeatAlreadyActuated,
PassNotReady,
}Expand description
A disabled select-then-actuate transition.
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.
SeatOutOfRange
The seat index is outside the configured seat universe.
CandidateOutOfRange
The candidate index is outside the configured candidate universe.
PassComplete
The pass has already committed its closure transition.
SeatAlreadyAllocated
The seat already has a selected allocation.
EffectAlreadyApplied
A score cannot change after the seat’s effect has been applied.
SeatNotAllocated
The seat has no selected allocation to actuate.
SeatAlreadyActuated
The seat’s selected allocation has already been actuated.
PassNotReady
At least one allocated seat still awaits actuation.