pub enum CommitPhase {
Pending,
Admitted,
Ready,
Retryable,
RecoveryPending,
Committed,
Rejected,
}Expand description
Concrete phase of the bounded governed-commit assembly.
Variants
Pending
Request has not yet been admitted.
Admitted
Capacity and registry admission have committed.
Ready
Propagation has prepared the request for its external effect.
Retryable
A pre-effect failure permits another bounded attempt.
RecoveryPending
An applied effect is waiting for durable recovery evidence.
Committed
Effect and durable evidence have both committed.
Rejected
The request cannot make another admitted attempt.